import dateutil.parser


class ClimacellResponse:
    """
    Wrapper around requests response object that includes a data() method that
    returns all ClimaCell specific data for the endpoint.

    Methods like json() and status_code are delegated to the requests response
    object initialized with the ClimacellResponse object
    """

    def __init__(self, request_response, fields, response_type='forecast'):
        self.request_response = request_response
        self.fields = fields
        self.response_type = response_type

    def data(self):
        raw_json = self.request_response.json()
        if self.status_code != 200:
            return ErrorData(raw_json)

        if self.response_type == 'realtime':
            return ObservationData(raw_json, self.fields)
        elif self.response_type == 'fire_index':
            return FireIndexData(raw_json)
        elif self.response_type == 'daily_forecast':
            observations = []
            for o_json in raw_json:
                observations.append(DailyObservationData(o_json, self.fields))
            return observations
        else:
            observations = []
            for o_json in raw_json:
                observations.append(ObservationData(o_json, self.fields))
            return observations

    def __getattr__(self, attrib):
        return getattr(self.request_response, attrib)


class ErrorData:

    def __init__(self, raw_json):
        self.raw_json = raw_json

    @property
    def error_code(self):
        return self.raw_json['errorCode']

    @property
    def error_message(self):
        return self.raw_json['message']


class ObservationData:

    def __init__(self, raw_json, fields):
        self.raw_json = raw_json
        self.fields = fields

    @property
    def lat(self):
        return self.raw_json.get('lat')

    @property
    def lon(self):
        return self.raw_json.get('lon')

    @property
    def observation_time(self):
        return dateutil.parser.parse(
                self.raw_json['observation_time']['value'])

    @property
    def measurements(self):
        m_dict = {}
        for f in self.fields:
            m_dict[f] = Measurement(
                    value=self.raw_json[f].get('value', None),
                    units=self.raw_json[f].get('units', None))
        return m_dict


class DailyObservationData(ObservationData):

    @property
    def measurements(self):
        m_dict = {}
        for f in self.fields:
            field_json = self.raw_json[f]
            if isinstance(field_json, list):
                m_dict[f] = {}
                for min_max in field_json:
                    key = 'max' if 'max' in min_max else 'min'
                    value = min_max[key].get('value', None)
                    units = min_max[key].get('units', None)
                    time = dateutil.parser.parse(min_max['observation_time'])
                    m_dict[f][key] = Measurement(value, units, time)
            else:
                m_dict[f] = Measurement(
                    value=field_json.get('value', None),
                    units=field_json.get('units', None))
        return m_dict


class FireIndexData:

    def __init__(self, raw_json):
        self.raw_json = raw_json

    @property
    def fire_index(self):
        return self.raw_json[0].get('fire_index', None)


class Measurement:

    def __init__(self, value, units, observation_time=None):
        self.value = value
        self.units = units
        self.observation_time = observation_time



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="climacell-python",
    version="0.0.6",
    description="Python library to interface with the ClimaCell Weather API",
    py_modules=["climacell_api"],
    packages=find_packages(include=['climacell_api'], exclude=['tests']),
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url="https://github.com/nicolo/climacell-python",
    author="Nicolo Giorgi",
    author_email="nicolo.giorgi@gmail.com",
    install_requires=[
            "requests >= 2.0",
            "python-dateutil >= 2.0",
        ],
    extras_require={
        "dev": [
            "pytest >= 5.0",
            "vcrpy >= 4.0",
            ],
    },
)



from datetime import datetime, timedelta, timezone
import dateutil.parser
import os
import vcr
from client import ClimacellApiClient

my_vcr = vcr.VCR(filter_query_parameters=[('apikey', 'CLIMACELL_API_KEY')])


@my_vcr.use_cassette('tests/vcr_cassettes/realtime-lat12-lon13.yml')
def test_real_time():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    response = api_client.realtime(
            lat='12', lon='13', units='us', fields=['wind_gust', 'temp'])

    assert response.status_code == 200
    data = response.data()
    assert data.lat == 12
    assert data.lon == 13
    assert data.observation_time == dateutil.parser.parse(
            '2020-06-22T20:44:29.185Z')
    measurements = data.measurements
    assert measurements['temp'].value == 96.46
    assert measurements['temp'].units == 'F'
    assert measurements['wind_gust'].value == 10.07
    assert measurements['wind_gust'].units == 'mph'
    assert response.json() == {
            'lat': 12,
            'lon': 13,
            'observation_time': {'value': '2020-06-22T20:44:29.185Z'},
            'temp': {'units': 'F', 'value': 96.46},
            'wind_gust': {'units': 'mph', 'value': 10.07}
            }


@my_vcr.use_cassette('tests/vcr_cassettes/realtime-bad-lon.yml')
def test_bad_params():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    response = api_client.realtime(
            lat='12', lon='999', fields=['wind_gust'])

    assert response.status_code == 400
    data = response.data()
    assert data.error_code == 'BadRequest'
    assert data.error_message == 'lon must be in the range -180..180'
    assert response.json() == {
            'statusCode': 400,
            'errorCode': 'BadRequest',
            'message': 'lon must be in the range -180..180',
            }


@my_vcr.use_cassette('tests/vcr_cassettes/nowcast-no-end-time.yml')
def test_nowcast():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    response = api_client.nowcast(
            lat='40', lon='80', timestep=30, start_time='now', units='us',
            fields=['wind_gust', 'precipitation_type'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 13
    assert data[0].lat == 40
    assert data[0].lon == 80
    assert data[0].observation_time == dateutil.parser.parse(
            '2020-06-22T20:47:00.196Z')
    measurements = data[0].measurements
    assert measurements['precipitation_type'].value == 'none'
    assert measurements['precipitation_type'].units is None
    assert measurements['wind_gust'].value == 13.42
    assert measurements['wind_gust'].units == 'mph'
    assert response.json()[0] == {
            'lat': 40,
            'lon': 80,
            'observation_time': {'value': '2020-06-22T20:47:00.196Z'},
            'wind_gust': {'units': 'mph', 'value': 13.42},
            'precipitation_type': {'value': 'none'}
            }


@my_vcr.use_cassette('tests/vcr_cassettes/nowcast-with-end-time.yml')
def test_nowcast_with_end_time():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    start_time = datetime(2020, 6, 22, 23, tzinfo=timezone.utc)
    end_time = start_time + timedelta(minutes=60)
    response = api_client.nowcast(
            lat='40', lon='80', timestep=30,
            start_time=start_time.isoformat(), end_time=end_time.isoformat(),
            fields=['wind_gust', 'precipitation_type'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 3
    assert data[0].lat == 40
    assert data[0].lon == 80
    assert data[0].observation_time == dateutil.parser.parse(
            '2020-06-22T23:00:00.000Z')
    measurements = data[0].measurements
    assert measurements['precipitation_type'].value == 'none'
    assert measurements['precipitation_type'].units is None
    assert measurements['wind_gust'].value == 3
    assert measurements['wind_gust'].units == 'm/s'
    assert response.json()[0] == {
            'lat': 40,
            'lon': 80,
            'observation_time': {'value': '2020-06-22T23:00:00.000Z'},
            'wind_gust': {'units': 'm/s', 'value': 3},
            'precipitation_type': {'value': 'none'}
            }


@my_vcr.use_cassette('tests/vcr_cassettes/forecast-hourly-no-end-time.yml')
def test_forecast_hourly():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    response = api_client.forecast_hourly(
            lat='43.08', lon='-89.54', start_time='now',
            fields=['wind_gust', 'precipitation_type'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 109
    assert data[0].lat == 43.08
    assert data[0].lon == -89.54
    assert data[0].observation_time == dateutil.parser.parse(
            '2020-06-22T20:00:00.000Z')
    measurements = data[0].measurements
    assert measurements['precipitation_type'].value == 'rain'
    assert measurements['precipitation_type'].units is None
    assert measurements['wind_gust'].value == 3.49
    assert measurements['wind_gust'].units == 'm/s'
    assert response.json()[0] == {
            'lat': 43.08,
            'lon': -89.54,
            'observation_time': {'value': '2020-06-22T20:00:00.000Z'},
            'wind_gust': {'units': 'm/s', 'value': 3.49},
            'precipitation_type': {'value': 'rain'}
            }


@my_vcr.use_cassette('tests/vcr_cassettes/forecast-hourly-with-end-time.yml')
def test_forecast_hourly_with_end_time():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    start_time = datetime(2020, 6, 22, 23, tzinfo=timezone.utc)
    end_time = start_time + timedelta(minutes=60)
    response = api_client.forecast_hourly(
            lat='40', lon='80',
            start_time=start_time.isoformat(), end_time=end_time.isoformat(),
            fields=['wind_gust', 'precipitation_type'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 2
    assert data[0].lat == 40
    assert data[0].lon == 80
    assert data[0].observation_time == dateutil.parser.parse(
            '2020-06-22T23:00:00.000Z')
    measurements = data[0].measurements
    assert measurements['precipitation_type'].value == 'none'
    assert measurements['precipitation_type'].units is None
    assert measurements['wind_gust'].value == 3
    assert measurements['wind_gust'].units == 'm/s'
    assert response.json()[0] == {
            'lat': 40,
            'lon': 80,
            'observation_time': {'value': '2020-06-22T23:00:00.000Z'},
            'wind_gust': {'units': 'm/s', 'value': 3},
            'precipitation_type': {'value': 'none'}
            }


@my_vcr.use_cassette('tests/vcr_cassettes/forecast-daily-no-end-time.yml')
def test_forecast_daily():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    response = api_client.forecast_daily(
            lat='43.08', lon='-89.54', start_time='now', units='us',
            fields=['temp', 'precipitation', 'sunrise'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 15
    assert data[0].lat == 43.08
    assert data[0].lon == -89.54
    assert data[0].observation_time == dateutil.parser.parse('2020-06-23')
    measurements = data[0].measurements
    assert measurements['temp']['min'].value == 52.25
    assert measurements['temp']['min'].units == 'F'
    assert(measurements['temp']['min'].observation_time ==
           dateutil.parser.parse('2020-06-23T11:00:00.000Z'))
    assert measurements['temp']['max'].value == 69.79
    assert measurements['temp']['max'].units == 'F'
    assert(measurements['temp']['max'].observation_time ==
           dateutil.parser.parse('2020-06-23T19:00:00.000Z'))
    assert measurements['precipitation']['max'].value == 0.0271
    assert measurements['precipitation']['max'].units == 'in/hr'
    assert(measurements['precipitation']['max'].observation_time ==
           dateutil.parser.parse('2020-06-23T22:00:00.000Z'))
    assert measurements['sunrise'].value == '2020-06-23T10:19:43.504Z'
    assert response.json()[0] == {
            "temp": [{
                "observation_time": "2020-06-23T11:00:00Z",
                "min": {
                    "value": 52.25,
                    "units": "F"
                }
                },
                {
                "observation_time": "2020-06-23T19:00:00Z",
                "max": {
                    "value": 69.79,
                    "units": "F"
                }
                }
            ],
            "precipitation": [{
                "observation_time": "2020-06-23T22:00:00Z",
                "max": {
                    "value": 0.0271,
                    "units": "in/hr"
                }
            }
            ],
            "sunrise": {
                "value": "2020-06-23T10:19:43.504Z"
            },
            "observation_time": {
                "value": "2020-06-23"
            },
            "lat": 43.08,
            "lon": -89.54,
        }


@my_vcr.use_cassette('tests/vcr_cassettes/forecast-daily-with-end-time.yml')
def test_forecast_daily_with_end_time():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    start_time = datetime(2020, 6, 24, 12, tzinfo=timezone.utc)
    end_time = start_time + timedelta(days=2)
    response = api_client.forecast_daily(
            lat='40', lon='80',
            start_time=start_time.isoformat(), end_time=end_time.isoformat(),
            fields=['temp', 'precipitation', 'sunrise'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 4
    assert data[0].lat == 40
    assert data[0].lon == 80
    assert data[0].observation_time == dateutil.parser.parse('2020-06-23')
    measurements = data[0].measurements
    assert measurements['temp']['min'].value == 26.75
    assert measurements['temp']['min'].units == 'C'
    assert(measurements['temp']['min'].observation_time ==
           dateutil.parser.parse('2020-06-23T23:00:00.000Z'))
    assert measurements['temp']['max'].value == 38.25
    assert measurements['temp']['max'].units == 'C'
    assert(measurements['temp']['max'].observation_time ==
           dateutil.parser.parse('2020-06-23T11:00:00.000Z'))
    assert measurements['precipitation']['max'].value == 0
    assert measurements['precipitation']['max'].units == 'mm/hr'
    assert(measurements['precipitation']['max'].observation_time ==
           dateutil.parser.parse('2020-06-23T07:00:00.000Z'))
    assert measurements['sunrise'].value == '2020-06-22T23:13:04.950Z'
    assert response.json()[0] == {
            "temp": [{
                "observation_time": "2020-06-23T23:00:00Z",
                "min": {
                    "value": 26.75,
                    "units": "C"
                }
                },
                {
                "observation_time": "2020-06-23T11:00:00Z",
                "max": {
                    "value": 38.25,
                    "units": "C"
                }
                }
            ],
            "precipitation": [{
                "observation_time": "2020-06-23T07:00:00Z",
                "max": {
                    "value": 0,
                    "units": "mm/hr"
                }
            }
            ],
            "sunrise": {
                "value": "2020-06-22T23:13:04.950Z"
            },
            "observation_time": {
                "value": "2020-06-23"
            },
            "lat": 40,
            "lon": 80
        }


@my_vcr.use_cassette('tests/vcr_cassettes/historical-climacell.yml')
def test_historical_climacell():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    start_time = datetime(2020, 6, 24, 12, tzinfo=timezone.utc)
    end_time = start_time + timedelta(hours=4)
    response = api_client.historical_climacell(
            lat='43.08', lon='-89.54', start_time=start_time, timestep=30,
            end_time=end_time,
            fields=['wind_gust', 'precipitation_type', 'sunset'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 9
    assert data[0].lat == 43.08
    assert data[0].lon == -89.54
    assert data[0].observation_time == dateutil.parser.parse(
            '2020-06-24T12:00:00.000Z')
    measurements = data[0].measurements
    assert measurements['precipitation_type'].value == 'none'
    assert measurements['precipitation_type'].units is None
    assert measurements['sunset'].value == '2020-06-25T01:41:48.098Z'
    assert measurements['sunset'].units is None
    assert measurements['wind_gust'].value == 7.5
    assert measurements['wind_gust'].units == 'm/s'
    assert response.json()[0] == {
            'lat': 43.08,
            'lon': -89.54,
            'observation_time': {'value': '2020-06-24T12:00:00.000Z'},
            'wind_gust': {'units': 'm/s', 'value': 7.5},
            'precipitation_type': {'value': 'none'},
            'sunset': {'value': '2020-06-25T01:41:48.098Z'}
            }


@my_vcr.use_cassette('tests/vcr_cassettes/historical-station.yml')
def test_historical_station():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    start_time = datetime(2020, 6, 23, 20, tzinfo=timezone.utc)
    end_time = start_time + timedelta(hours=4)
    response = api_client.historical_station(
            lat='43.08', lon='-89.54', start_time=start_time,
            end_time=end_time, fields=['temp', 'precipitation_type'])

    assert response.status_code == 200
    data = response.data()
    assert len(data) == 12
    assert data[0].observation_time == dateutil.parser.parse(
            '2020-06-23T20:15:00.000Z')
    measurements = data[0].measurements
    assert measurements['precipitation_type'].value == 'none'
    assert measurements['precipitation_type'].units is None
    assert measurements['temp'].value == 22.4
    assert measurements['temp'].units == 'C'
    assert response.json()[0] == {
            'observation_time': {'value': '2020-06-23T20:15:00.000Z'},
            'temp': {'units': 'C', 'value': 22.4},
            'precipitation_type': {'value': 'none'}
            }


@my_vcr.use_cassette('tests/vcr_cassettes/insights-fire-index.yml')
def test_insights_fireindex():
    api_client = ClimacellApiClient(key=os.getenv('CLIMACELL_KEY'))
    response = api_client.insights_fire_index(lat=43.08, lon=-89.54)

    assert response.status_code == 200
    data = response.data()
    assert data.fire_index == 30.474195
    assert response.json() == [{'fire_index': 30.474195}]
